@@ -29,6 +29,7 @@ tag_singular = 0b00101111
 tag_kill_mark = 0b100000001111
 tag_void = 0b0000000000001111
 tag_seek_mark = 0b1000000000001111 | 1<<63
+tag_move = 0b1100000000001111
 tag_box_header = 0b0100000000001111
 
 btag_bignum   = 0x40000000
@@ -45,6 +46,7 @@ mask_boolean = 0xff
 mask_kill_mark = 0xfff
 mask_void = 0xffff
 mask_seek_mark = 0xffff | 1<<63
+mask_move = 0xffff
 mask_box_header = 0xffff
 
 mask_boxhdr_length = 0xffffffff00000000
@@ -57,6 +59,7 @@ shift_char = 8
 shift_boolean = 8
 shift_singular = 8
 shift_void = 16
+shift_move = 16
 shift_boxhdr_value = 16
 shift_flonum = 32
 
@@ -97,6 +100,11 @@ class LokoVoidPrinter(LokoPrinter):
         return "#<void #x%x>" % (self.val >> shift_void)
 
 
+class LokoMovePrinter(LokoPrinter):
+    def to_string(self):
+        return "#<move #x%x>" % (self.val >> shift_void)
+
+
 class LokoPairPrinter(LokoPrinter):
     def to_string(self):
         car = deref(self.val, 0)
@@ -376,8 +384,6 @@ def loko_lookup_function(val):
                 return LokoSingularPrinter('()')
             if stype == 1:
                 return LokoSingularPrinter('#!eof')
-            if stype == 2:
-                return LokoSingularPrinter('#!moved')
         if tag8 == tag_boolean:
             value = int(val) >> shift_boolean
             if value == 0:
@@ -392,6 +398,8 @@ def loko_lookup_function(val):
         tag16 = int(val) & 0xffff
         if tag16 == tag_void:
             return LokoVoidPrinter(val)
+        if tag16 == tag_move:
+            return LokoMovePrinter(val)
 
         return LokoPrinter(val)
 
