@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- mode: python; coding: utf-8 -*-
-# Copyright © 2018, 2019, 2021 Göran Weinholt <goran@weinholt.se>
+# Copyright © 2018-2022 Göran Weinholt <goran@weinholt.se>
 # SPDX-License-Identifier: MIT
 
 # Search for or prove Loko's type tagging and its properties.
@@ -23,13 +23,14 @@ tag_kill_mark  = BitVec('tag-kill-mark', 64)
 tag_void       = BitVec('tag-void', 64)
 tag_flonum     = BitVec('tag-flonum', 64)
 tag_seek_mark  = BitVec('tag-seek-mark', 64)
+tag_move       = BitVec('tag-move', 64)
 tag_singular   = BitVec('tag-singular', 64)
 tag_box_header = BitVec('tag-box-header', 64)  # first word in a box
 
 tags = (
     tag_fixnum, tag_box, tag_pair, tag_procedure, tag_string, tag_vector, tag_bytevector,
     tag_immsym, tag_char, tag_boolean, tag_kill_mark, tag_void, tag_flonum, tag_seek_mark,
-    tag_singular, tag_box_header
+    tag_move, tag_singular, tag_box_header
 )
 
 mask_fixnum     = BitVecVal(0b00000111, 64)
@@ -46,13 +47,14 @@ mask_kill_mark  = BitVec('mask-kill-mark', 64)
 mask_void       = BitVec('mask-void', 64)
 mask_flonum     = BitVec('mask-flonum', 64)
 mask_seek_mark  = BitVec('mask-seek-mark', 64)
+mask_move       = BitVec('mask-move', 64)
 mask_singular   = BitVec('mask-singular', 64)
 mask_box_header = BitVec('mask-box-header', 64)
 
 masks = (
     mask_fixnum, mask_box, mask_pair, mask_procedure, mask_string, mask_vector, mask_bytevector,
     mask_immsym, mask_char, mask_boolean, mask_kill_mark, mask_void, mask_flonum, mask_seek_mark,
-    mask_singular, mask_box_header
+    mask_move, mask_singular, mask_box_header
 )
 
 # The first word in a box may be a box header. It has a type, length
@@ -117,6 +119,7 @@ s.add(tag_kill_mark      == 0b100000001111)
 s.add(tag_void       == 0b0000000000001111)
 s.add(tag_seek_mark  == 0b1000000000001111 | 1<<63)
 s.add(tag_box_header == 0b0100000000001111)
+s.add(tag_move       == 0b1100000000001111)
 
 s.add(btag_bignum   == 0x40000000)
 s.add(btag_ratnum   == 0x43000000)
@@ -134,6 +137,7 @@ s.add(mask_boolean      == 0xff)
 s.add(mask_kill_mark   == 0xfff)
 s.add(mask_void       == 0xffff)
 s.add(mask_seek_mark  == 0xffff | 1<<63)
+s.add(mask_move       == 0xffff)
 s.add(mask_box_header == 0xffff)
 
 s.add(mask_boxhdr_length        == 0xffffffff00000000)
@@ -147,7 +151,7 @@ imm_tag  = BitVecVal(0b111, 64)
 imm_mask = BitVecVal(0b111, 64)
 s.add([(tag & imm_mask) == imm_tag for tag in
        (tag_immsym, tag_char, tag_boolean, tag_kill_mark,
-        tag_void, tag_flonum, tag_seek_mark,
+        tag_void, tag_flonum, tag_seek_mark, tag_move,
         tag_singular, tag_box_header)])
 
 # Booleans can be created with setcc ah
@@ -166,6 +170,9 @@ s.add([And(mask > 0, mask <= 0xffff) for mask in masks if mask is not mask_seek_
 # s.add([(tag & (mask_boxhdr_length | mask_boxhdr_refs_p | mask_boxhdr_value)) == 0
 #        for tag in btags])
 
+# The move mark can encode a 48-bit pointer
+s.add(0xFFFFFFFFFFFF0000 & ~mask_move == 0xFFFFFFFFFFFF0000)
+
 # Tags fit inside their bitmasks
 s.add([(tag & mask) == tag for (tag, mask) in zip(tags, masks)])
 s.add([(tag & mask_boxhdr_type) == tag for tag in btags])
